/********************************************************
 Persistence of Vision Ray Tracer Scene Description File
 File       : sample.pov
 BM_Version : 4.0
 Description: Sample Scene using Blob Man Figures
 Date       : April 2000 
 Author     : Peter Houston 
 Email      : houston.graphics@iname.com
 WWW        : http://welcome.to/HoustonGraphics
*********************************************************/

global_settings {  assumed_gamma 2.2 }

camera {  
    location  <0, 2, -5>
    direction 1.5*z
    right     4/3*x
    look_at   <-.25, 1, 0> 
}


sky_sphere {
    pigment {
        gradient y
        color_map { [0.0 color rgb <0.7,0.7,1.0>] [1.0 color blue 0.5] }
    }
}

light_source {
    <-20, 800, -800>
    color rgb <1,.9,.95>*1.5
    spotlight               
    point_at <0, 0, 0>      
    radius .1               
    tightness 75            
    falloff 3               
}

plane {y,0
  pigment {checker pigment {rgb 0}, pigment {rgb 1} rotate y*16}
}

#declare Hair=on; // Warning : Turning ppHair on increase parsing time !

#include "bmpeople.inc"
#include "bm_pose.inc"
#include "bmtorso.inc"
#include "bmhand.inc"
#include "bmfeet.inc"

#declare BM_EyeLids=<.35,0,.35>;
#declare BM_Jaw=.25;
#declare BM_Mouth=1;
#declare BM_MouthShape=1;



// Reclined Female
#declare BM_PoseNo=11;      
Blob_Man(Female,1.8) 

#declare ppSkinDarkness=1;
#declare ppSkinRedness=1;                      
#declare ppHairStyle = 2;                       
#declare ppHairColour = <0.55,0.45,0.3>;        // Brunette
#declare ppEyeshadowExtent = 0.5;               // 0.2 to 1.5     1 is default
#include "HeadforBlobMan.inc"  

#declare BlobMan= union {   // Rebuild BlobMan with new ppHead
  object {BHead}
  object {BNeck}
  object { bmtorso(  )}
  object { bmhand ( 1)}
  object { bmhand (-1)}
  object { bmfeet (  )}
}   

object {BlobMan transform BMO_Abdomen_L rotate z*-85}


// Female Standing to Attention                             
#declare BM_PoseNo=0;
Blob_Man(Female,1.8)

#declare ppSkinDarkness=1;
#declare ppSkinRedness=1;
#declare ppHairColour = <1,0.9,0.6>;      // Golden
#declare ppHairStyle = 1;                 // See hairstyle.inc for valid values
#declare ppHairThickness = 0.03;
#declare ppHairNumber = 4000;             // 12000 gives good cover, but 1 hour render time. 1000 gives enough for test renders
#declare ppHairFringeLength = 1.7;
#declare ppLashLength    = 30;                    // 10    to 30 
#declare ppEyeshadowColour  = <0.25,0.3,0.25,0.8>; // More or less any colour you want
#include "HeadforBlobMan.inc"  

#declare BlobMan= union {   // Rebuild BlobMan with new ppHead
  object {BHead}
  object {BNeck}
  object { bmtorso(  )}
  object { bmhand ( 1)}
  object { bmhand (-1)}
  object { bmfeet (  )}
}   

object {BlobMan transform BMO_Foot_L translate <-.5,0,1>}
 

// Blobman Male in Lotus Position with POV-Person Female Head
#declare BM_PoseNo=2;
Blob_Man(Male,2)

#declare ppSkinDarkness=1.2;
#declare ppSkinRedness=0.9;
#declare ppHairColour = <0.9,0.85,0.65>;        // Blonde
#declare ppHairStyle = 5;                 //5 See hairstyle.inc for valid values
#declare ppHairNumber = 7000;             // 12000 gives good cover, but 1 hour render time. 1000 gives enough for test renders
#declare ppHairFringeLength = 1.2;
#include "HeadforBlobMan.inc"  

#declare BlobMan= union {   // Rebuild BlobMan with new ppHead
  object {BHead}
  object {BNeck}
  object { bmtorso(  )}
  object { bmhand ( 1)}
  object { bmhand (-1)}
  object { bmfeet (  )}
}   

object {BlobMan transform BMO_Abdomen_U rotate y*45 translate <1,0,-.5>}                             


// Blobman Male Performing a Hand Stand (with POV-Person Mohecan Hair)
#declare BM_PoseNo=7;
Blob_Man(Male,1.85)

#declare ppSkinDarkness=1;
#declare ppSkinRedness=1;
#declare ppHairColour = <0.3,0.2,0.2>;      // Black
#declare ppHairStyle = 3;                 // See hairstyle.inc for valid values
#declare ppHairThickness = 0.03;
#declare ppHairNumber = 4000;             // 12000 gives good cover, but 1 hour render time. 1000 gives enough for test renders
#declare ppHairFringeLength = 1.7;
#declare ppLashLength    = 30;                    // 10    to 30 
#declare ppEyeshadowColour  = <0.25,0.3,0.25,0.8>; // More or less any colour you want
#include "HeadforBlobMan.inc"  

#declare BlobMan= union {   // Rebuild BlobMan with new ppHead
  object { bmppHead()}
  object {shortppHair scale 0.11 translate <0,-9.3,-2> 
    translate <0,5,-1>
    rotate z*BM_Head_Rot.z
    rotate x*BM_Head_Rot.x
    rotate y*BM_Head_Rot.y
    transform {rotate <BM_Neck_Rot.x,0,BM_Neck_Rot.z>}
    translate <0,-5,1>
    transform {translate <0,20,-3> rotate <BM_Torso_Rot.x,0,BM_Torso_Rot.z> translate <0,-20,3>
                                   translate <0,20,0>  rotate <0,BM_Torso_Rot.y,0> translate <0,-20,0>}
  }
  object { bmtorso(  )}
  object { bmhand ( 1)}
  object { bmhand (-1)}
  object { bmfeet (  )}
}   

object {BlobMan transform BMO_Hand_R translate <-1,0,0>}


// Female "We're not worthy" 
#declare BM_PoseNo=6;
Blob_Man(Female,1.85)

#declare ppSkinDarkness=1;
#declare ppSkinRedness=1;
#declare ppHairColour = <0.55,0.45,0.3>;        // Brunette
#declare ppHairStyle = 1;                 // See hairstyle.inc for valid values
#declare ppHairThickness = 0.009;
#declare ppHairNumber = 4000;             // 12000 gives good cover, but 1 hour render time. 1000 gives enough for test renders
#declare ppHairFringeLength = 0.7;
#declare ppEyeshadowColour  = <0.25,0.4,0.15,0.2>; // More or less any colour you want
#declare ppLipColour = <1,0.5,0.5,0.3>;            // 'Normal' Red ppLips 

#include "HeadforBlobMan.inc"  

#declare BlobMan= union {   // Rebuild BlobMan with new ppHead
  object {BHead}
  object {BNeck}
  object { bmtorso(  )}
  object { bmhand ( 1)}
  object { bmhand (-1)}
  object { bmfeet (  )}
}   

object {BlobMan transform BMO_Knee_R rotate y*-45 translate <0,1,0>*BM_Scale translate <-1.5,0,-1>}                      
  

  
// Female Sunbathing
#declare BM_HairStyle=1*Hair;
#declare BM_PoseNo=8;
Blob_Man(Female,2)
object {BlobMan transform BMO_Torso_B rotate x*99 rotate y*30 translate <1,0,2.5>} 
 
// EOF
