/***************************************************************************************************/
/* Head.pov  - Copyright Chris Bartlett 7 May 2004                                                 */
/* This work is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike License.  */
/* To view a copy of this license, visit http://creativecommons.org/licenses/by-nc-sa/1.0/ or      */
/* send a letter to Creative Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.      */
/***************************************************************************************************/

#declare headRotation = -y*25;

#declare ppCamera = "Here"; 
camera {
  location <-0, 90, -320>
  look_at <35, 60, 0>
  angle 34.3775
}

light_source { <250, 60,-8900> color rgb<1, 1, 1>*0.6 }   
light_source { <0, 1400,-1600> color rgb<1, 1, 1>*1.2 }   
light_source { <0, 60,0> color rgb<1, 0.5, 0.5>}   


/*
plane {y,8
  pigment {color rgb <0.5,0.7,0.7>}  normal {bumps 0.4 scale 100}
  translate z*1000
}
*/
                                                // Recommended Values
                                                // ******************
#declare ppSkinDarkness=1;                        // 0.7 to 8       1 is default, 2 is more dark
#declare ppSkinRedness=1;                         // 0.9 to 2       1 is not very red, 1.5 is redder  
#declare ppSkinNoiseGenerator=3;                  // See noise_generator in the help

#declare ppLipColour = <1,0,0.1,0.4>;              // The Transparency defines how much affect the skin colour has on the lip colour  
#declare ppEyeshadowColour  = <0.25,0.3,0.25,0.2>; // More or less any colour you want
#declare ppEyeshadowExtent = 0.3;                 // 0.2 to 1.5     1 is default
#declare ppBlusherColour  = <1,0,0,0.5>;           // More or less any colour you want
#declare ppBlusherExtent = 0.4;                   // 0.3 to 1       0.4 is default

#declare ppLashLength    = 25;                    // 10    to 30 
#declare ppLashDensity   = 0.6;                   // 0.5   to 1 
#declare ppLashThickness = 0.016;                 // 0.009 to 0.036 
#declare ppLashColour    = <0,0,0>;               //
   
#declare ppEyebrowHairThickness = 0.030;          // 0.009 to 0.036 
#declare ppEyebrowHairColour    = <0.1,0.1,0.1,0.3>;   
          
#declare ppEyeColour = 1;                         //  1=Blue, 2=Brown, 3=Green
#declare ppEyeBrightness = 1;                     // 0.5 = Dark, 1.5 = Bright    
#declare ppEyePupil = 1;                          // 1.35 = dilated (big black bit), 0.35 = contracted (small black bit) 
#declare ppEyePosition = <13.2, 80.6, -3.5>;      //

#declare characterPrefix = "f1";
#include "ppHeadTextures.inc"  
#include "ppupperteeth.inc"
#include "pplowerteeth.inc"
#include "ppEye.inc"
#include "ppuppereyelash.inc"
#include "pplowereyelash.inc"
#include "ppf1eyebrow.inc"
#include concat("pp",characterPrefix,"ear.inc")
#include concat("pp",characterPrefix,"face.inc")
#include concat("pp",characterPrefix,"lips.inc")

#declare ppSkin = union {
  object {ppFace}
  object {ppFace scale <-1,1,1>}
  object {ppEar}   
  object {ppEar scale <-1,1,1>} 
  texture {ppAltSkinTexture_3}          //  Skin texture spiral ** Must be used in isolation ** 
} 
  
#declare ppHead = union {

  object {ppSkin}

  object {ppLips texture {ppLipTexture}}
  object {ppLips texture {ppLipTexture} scale <-1,1,1>}
  object {ppEye1 rotate <0,0,0> translate ppEyePosition}
  object {ppEye1 rotate <0,0,0> translate ppEyePosition scale <-1,1,1>}  

  object {ppUpperLash texture {ppEyelashtexture translate ppEyePosition-2*z}}
  object {ppUpperLash texture {ppEyelashtexture translate ppEyePosition-2*z} scale <-1,1,1>}
  object {ppLowerLash texture {ppEyelashtexture scale <1,-1,1> translate ppEyePosition-1*z}}
  object {ppLowerLash texture {ppEyelashtexture scale <1,-1,1> translate ppEyePosition-1*z} scale <-1,1,1>}

  object {ppEyeBrow texture {ppEyebrowTexture  rotate -z*45 translate ppEyePosition+<4,-4,10>}}
  object {ppEyeBrow texture {ppEyebrowTexture  rotate -z*45 translate ppEyePosition+<4,-4,10>} scale <-1,1,1>}  

  object {ppUpperTeeth pigment {color rgb <1,1,1>} translate -x }
  object {ppUpperTeeth pigment {color rgb <1,1,1>} translate -x scale <-1,1,1>}
  object {ppLowerTeeth pigment {color rgb <1,1,1>} translate -x }
  object {ppLowerTeeth pigment {color rgb <1,1,1>} translate -x scale <-1,1,1>}

}
                                 
object {ppHead rotate headRotation}   

            
plane {y,0  
  texture {
    pigment {color rgbf <1,1,1,1>} 
    finish {reflection 0.6 phong 1}
    normal {bumps 0.4 scale 0.3} 
    scale 500
  }
}

sphere {<180,0,800>,80   // moon
  texture {
    pigment {bozo
      color_map { 
        [0    color rgb <1,1,1>]
        [0.8  color rgb <0.3,0.3,0.3>]
        [1    color rgb <1,1,1>]
      }
    }
    finish {ambient 1}
    scale 5  
  }
  clipped_by { plane {-y,-0.5}}
  scale 500
}

sphere {<0,0,0>,1000  // stars
  texture {
    pigment {spotted 
      color_map { 
        [0    color rgb <1,1,1>]
        [0.05 color rgb <0,0,0>]
        [1    color rgb <0,0,0>]
      }
    }
    finish {ambient 1} 
    scale 0.002
  }
  clipped_by { plane {-y,-0.5}}
  scale 500
}

sphere {<0,0,0>,700   // clouds
  texture {
    pigment {spotted turbulence 1.8
      color_map { 
        [0    color rgbf <0.5,0.5,0.5,0.2>]
        [0.1  color rgbf <0.5,0.5,0.5,0>]
        [0.25 color rgbf <1,1,1,1>]
        [1    color rgbf <1,1,1,1>]
      }
    }
    finish {ambient 0.2}
    scale <500,50,500>
    scale 0.002
  }
  clipped_by { plane {-y,-0.5}}
  scale 500
}
