// ISSUES
// Heel orientation is not always correct, depending on the angle of the boot.
// Toe of boot needs defining for sole
// ppAddOffSetToArray probably won't animate correctly

#include "math.inc"
#include "ppclothingmacros.inc" 

// This macro generates a single boot by wrapping material around a series of objects representing elements of a leg.
//
// Parameters:
// -----------
//               - No parameters are passed to this macro. 
//
#macro ppBootGenerator ()
  #ifndef(ppVerbose)    #declare ppVerbose = 0; #end
  #ifndef(ppShowPoints) #declare ppShowPoints = 0; #end
  
  //**************************************//
  // Default values for control variables //
  //**************************************//
  // Define the 'looseness' of the boots
  #ifndef (ppUpperOffSet)   #declare ppUpperOffSet = 0.012;      #end    // Distance from the skin for smooth sections
  #ifndef (ppBootBumpiness) #declare ppBootBumpiness = 0.045;    #end    // Distance from the skin for bumpy sections
  
  // The number of slices determine the quality of each part 
  #ifndef (ppToeSlices  ) #declare ppToeSlices   = 5;   #end  // Warning. This one does a bit more than that. Only alter it if you know what you're doing.
  #ifndef (ppUpperSlices) #declare ppUpperSlices = 8;   #end
  #ifndef (ppHeelSlices ) #declare ppHeelSlices  = 10;  #end
  #ifndef (ppCoreSlices ) #declare ppCoreSlices  = 18;  #end
  #ifndef (ppRimSlices  ) #declare ppRimSlices   = 2;   #end
  
  // Dimensions of the boot
  #ifndef (ppRimThickness     ) #declare ppRimThickness      = 0.004;            #end
  #ifndef (ppHeelHeight       ) #declare ppHeelHeight        = 0.09;             #end
  #ifndef (ppHeelWidth        ) #declare ppHeelWidth         = ppHeelHeight*0.9; #end
  #ifndef (ppSoleSlices       ) #declare ppSoleSlices        = 12;               #end
  #ifndef (ppSoleToeSlices    ) #declare ppSoleToeSlices     = 10;               #end
  #ifndef (ppSoleSegments     ) #declare ppSoleSegments      = 10;               #end
  #ifndef (ppSoleThickness    ) #declare ppSoleThickness     = 0.006;            #end
  #ifndef (ppPlatformThickness) #declare ppPlatformThickness = 0.02;             #end
  #ifndef (ppZipFlapWidth     ) #declare ppZipFlapWidth      = 0.01;             #end
  #ifndef (ppZipFlapThickness ) #declare ppZipFlapThickness  = 0.0012;           #end
  
  // The number of segments should be consistent to contribute to seamless assembly of component parts.
  #ifndef(ppBootSegments)       #declare ppBootSegments      = 15;               #end
  
  // A spotted pigment is used to give texture to the part of the boot covering the leg.
  #ifndef (ppBootDisplacementPigment) #declare ppBootDisplacementPigment = pigment {spotted scale <0.2,0.02,0.2>*0.3} #end 
  #ifndef (ppBootColour)          #declare ppBootColour          = <0.9,0.6,0.3>; #end
  #ifndef (ppBootSoleColour)      #declare ppBootSoleColour      = <0,0,0>;       #end
  #ifndef (ppBootInnerHeelColour) #declare ppBootInnerHeelColour = <1,0,0>;       #end
  
  #ifndef (ppBootLegTexture) 
    #declare ppBootLegTexture = texture {
      pigment {color rgb ppBootColour}  
      normal {bumps 0.8 scale <0.1,0.03,0.1>} 
      finish {phong 0.9}
      scale 0.5
    }
  #end
  #ifndef (ppBootUpperTexture) 
    #declare ppBootUpperTexture = texture {
      pigment {color rgb ppBootColour}            //  <0.3,0.2,0.1>
      finish {phong 0.9}
    }
  #end
  #ifndef (ppBootSoleTexture) 
    #declare ppBootSoleTexture = texture {
      pigment {color rgb ppBootSoleColour}
      finish {phong 0.9}
    }
  #end
  
  #ifndef (ppBootToeTexture      ) #declare ppBootToeTexture       = texture {ppBootUpperTexture};                         #end
  #ifndef (ppBootHeelOuterTexture) #declare ppBootHeelOuterTexture = texture {ppBootUpperTexture};                         #end 
  #ifndef (ppBootHeelInnerTexture) #declare ppBootHeelInnerTexture = texture {pigment {color rgb ppBootInnerHeelColour}};  #end
  #ifndef (ppBootRimTexture      ) #declare ppBootRimTexture       = texture {ppBootUpperTexture};                         #end
  #ifndef (ppBootZipTexture      ) #declare ppBootZipTexture       = texture {ppBootRimTexture};                           #end
  
  
  #ifndef (ppBootTexture2) 
    #declare ppBootTexture2 = texture {
      pigment {color rgb ppBootColour}            //  <0.3,0.2,0.1>
      finish {phong 0.9}
    }
  #end
  
  //*******************//
  // Working variables //
  //*******************//
  #local ppSegments = ppBootSegments;
  
  // Arrays to store surface positions and normals
  #local ppBootToeArray      = array [ppToeSlices][ppSegments];
  #local ppBootToeNormal     = array [ppToeSlices][ppSegments];  
  #local ppBootUpperArray    = array [ppUpperSlices][ppSegments];
  #local ppBootUpperNormal   = array [ppUpperSlices][ppSegments];  
  #local ppBootHeelArray     = array [ppHeelSlices+1][ppSegments];
  #local ppBootHeelNormal    = array [ppHeelSlices+1][ppSegments];  
  #local ppBootCoreArray     = array [ppCoreSlices][ppSegments];
  #local ppBootCoreNormal    = array [ppCoreSlices][ppSegments];  
  #local ppBootRimArray      = array [4][ppSegments];
  #local ppBootRimNormal     = array [4][ppSegments];  
  #local ppBootSoleArray     = array [ppSoleSlices][ppSoleSegments];
  #local ppBootSoleNormal    = array [ppSoleSlices][ppSoleSegments];  
  #local ppBootSoleToeArray  = array [ppSoleToeSlices][ppSoleSegments];
  #local ppBootSoleToeNormal = array [ppSoleToeSlices][ppSoleSegments];  
  
  #local ppRefLength  = vlength(ppFootStartPos-ppCalfStartPos);
  
  #local ppFootRefVector = vrotate(ppCalfStartPos-ppCalfEndPos,ppFootOrientation);
  #local ppHeelCentre    = ppCalfStartPos+ppRefLength*0.33*ppFootRefVector; 
  
  // The upper (covering the surface of the foot)
  #local ppUpperStartPos    = ppFootStartPos;
  #local ppUpperStartVector = -vrotate(z,ppMiddleToeOrientation);;
  #local ppUpperEndPos      = ppUpperStartPos + 0.5*(ppHeelCentre-ppUpperStartPos);
  #local ppUpperEndVector   = vnormalize(ppCalfStartPos-ppUpperEndPos);
  // The fabric over the heel of the boot - We pick up the start position and vector from where we left the end of the boot upper
  #local ppHeelStartPos     = ppUpperEndPos;
  #local ppHeelStartVector  = ppUpperEndVector;
  #local ppHeelEndPos       = ppCalfStartPos-0.3*(ppCalfStartPos-ppHeelStartPos);
  #local ppHeelEndVector    = vnormalize(ppCalfStartPos-(ppUpperStartPos + 0.95*(ppHeelCentre-ppUpperStartPos)));
  // The fabric around the leg
  #local ppCoreStartPos     = ppCalfStartPos+0.025*(ppCalfEndPos-ppCalfStartPos);
  #local ppCoreEndPos       = ppCalfStartPos+0.8  *(ppCalfEndPos-ppCalfStartPos);
  // The rim around the top of the boot
  #local ppRimStartPos      = ppCoreEndPos;
  #local ppRimEndPos        = ppCoreEndPos + 0.02*vnormalize(ppCoreEndPos-ppCoreStartPos);
   
  #if (strcmp(ppHandedness,"Left")=0)
    #local ppRotationDirection = 1;
  #else  
    #local ppRotationDirection = -1;
  #end
  
  // The reference vector is used to consistently orient the starting point on each circular scan.
  // It contributes to the ease with which various generated pieces can be seamlessly assembled.
  // It should be perpendicular to the general flow of the lines through the inside of the set of
  // objects. So we use a vector that is perpendicular to two of the lines used as the centre lines
  // for two of the scans. 
  #local ppRefVector     = VPerp_To_Plane(ppUpperEndPos - ppUpperStartPos, ppCoreEndPos - ppCoreStartPos)*ppRotationDirection;
  #local ppPerpRefVector = vaxis_rotate(ppRefVector,(ppUpperEndPos - ppUpperStartPos),ppRotationDirection*90);
  #local ppTraceLength   = 0.15;
  #local ppJoinEnds      = 1;
  
  // We need to calculate the angles required to orient the bumpy texture to the line of the leg
  #local ppLegVector    = ppCoreEndPos - ppCoreStartPos;
  #local ppLegTempVector= VProject_Plane(ppLegVector, z);
  #local ppXRot         = VAngleD(ppLegTempVector,ppLegVector);
  #if (ppLegVector.z<0) #local ppXRot = -ppXRot; #end
  #local ppZRot         = VAngleD(y,ppLegTempVector);
  #if (ppLegVector.x<0) #local ppZRot = -ppZRot; #end
  
  
  //difference {  object {ppFootObject} plane {-y,0 pigment {color rgbt 1} translate <-0.000,0.015,0.166>}}
  
  // Generate the Toe of the Boot
  #if (ppVerbose = 1) #debug concat("Start of Boot Toe","\n")  #end
  #local ppI = 0;
  #while (ppI<5)
    #local ppJ = 0;
    #while (ppJ<ppSegments)
      #local ppCentreIndex = min(ppI,2);
      #if (ppJ<2|ppJ>13) // Round the big toe  
        #local ppRayDirection = ppRefVector;
        #if (ppI=3) 
          #local ppRayDirection = vaxis_rotate(ppRayDirection,ppPerpRefVector,ppRotationDirection*60);
        #end  
        #if (ppI=4) 
          #local ppRayDirection = vaxis_rotate(ppRayDirection,ppPerpRefVector,ppRotationDirection*90);
        #end  
        #local ppSliceCentre = ppToeCentreLines[ppCentreIndex][0];
        #if (ppJ=14) #local ppRayDirection = vaxis_rotate(ppRayDirection,(ppUpperEndPos - ppUpperStartPos),ppRotationDirection*45); 
        #else #local ppRayDirection = vaxis_rotate(ppRayDirection,(ppUpperEndPos - ppUpperStartPos),-ppRotationDirection*45*ppJ); 
        #end 
      #end
      #if (ppJ>1&ppJ<7) // Top surface of toes
        #local ppSliceCentre = ppToeCentreLines[ppCentreIndex][ppJ-2];
        #local ppRayDirection = -ppFootRefVector; //-ppPerpRefVector;
        #if (ppI=3) 
          #local ppRayDirection = vaxis_rotate(ppRayDirection,ppRefVector,ppRotationDirection*60);
        #end  
        #if (ppI=4) 
          #local ppRayDirection = vaxis_rotate(ppRayDirection,ppRefVector,ppRotationDirection*90);
        #end  
      #end
      #if (ppJ>6&ppJ<9) // Round the little toe
        #local ppRayDirection = ppRefVector;
        #if (ppI=3) 
          #local ppRayDirection = vaxis_rotate(ppRayDirection,ppPerpRefVector,-ppRotationDirection*60);
        #end  
        #if (ppI=4) 
          #local ppRayDirection = vaxis_rotate(ppRayDirection,ppPerpRefVector,-ppRotationDirection*90);
        #end  
        #local ppSliceCentre = ppToeCentreLines[ppCentreIndex][4];
        #local ppRayDirection = vaxis_rotate(-ppRayDirection,(ppUpperEndPos - ppUpperStartPos),ppRotationDirection*(90-60*(ppJ-6))); 
      #end
      #if (ppJ>8&ppJ<14) // Bottom surface of toes
        #local ppSliceCentre = ppToeCentreLines[ppCentreIndex][13-ppJ];
        #local ppRayDirection = ppFootRefVector; 
        #if (ppI=3) 
          #local ppRayDirection = vaxis_rotate(ppRayDirection,ppRefVector,-ppRotationDirection*60);
        #end  
        #if (ppI=4) 
          #local ppRayDirection = vaxis_rotate(ppRayDirection,ppRefVector,-ppRotationDirection*90);
        #end  
      #end
      #local ppNorm = <0, 0, 0>; 
      #local ppThisPoint = trace(ppFootObject, ppSliceCentre+ppRayDirection, -ppRayDirection, ppNorm);
      #if (vlength(ppNorm)!=0) 
        #local ppBootToeArray[ppI][ppJ] = ppThisPoint;
        #local ppBootToeNormal[ppI][ppJ] = ppRayDirection;
      #else
        #local ppBootToeArray[ppI][ppJ] = <0,0,0>;
        #local ppBootToeNormal[ppI][ppJ] = <0,1,0>;
        #warning concat("The Toe trace sent out a ray for ",str(ppI,3,0),":",str(ppJ,3,0),"from point <",vstr(3,ppSliceCentre,",",0,3),"> that didn't intercept the object.\n")
      #end 
      #local ppJ = ppJ + 1;
    #end
    #local ppI = ppI + 1;
  #end 
  //#local ppTemp = vrotate(z,ppBodyPartListOrientation[ppLeftLittleToeTipRef]);
  //cylinder {ppBodyPartListPosition[ppLeftLittleToeTipRef],ppBodyPartListPosition[ppLeftLittleToeTipRef]+ppTemp,0.0005 pigment {color rgb <1,2,0>}}
  ppAddOffSetToArray(0, ppToeSlices-1, 0, ppSegments-1, ppBootToeArray, ppBootToeNormal, ppBootDisplacementPigment,0,ppUpperOffSet*1.1)
  #declare ppBoot1Toe = ppMeshFromArray(ppBootToeArray,ppBootToeNormal,ppToeSlices,ppSegments,ppJoinEnds)
  #if (ppShowPoints=1) ppDrawPoints(ppBootToeArray,ppToeSlices,ppSegments,0.001,<0,1,1>) #end 
  
  // Generate the Upper of the Boot (covering the surface of the foot)
  #if (ppVerbose = 1) #debug concat("Start of Boot Upper","\n")  #end 
  ppLimbScan(ppFootObject,ppUpperStartPos,ppUpperStartVector,ppRefVector,ppUpperEndPos,ppUpperEndVector,ppRefVector,ppUpperSlices-1,
             ppSegments,ppBootUpperArray,ppBootUpperNormal,1,ppRotationDirection,ppTraceLength)
  ppStitchArrays(ppBootToeArray,ppBootToeNormal, 0, ppBootUpperArray,ppBootUpperNormal,0,ppSegments)
  ppAddOffSetToArray(1, ppUpperSlices-1, 0, ppSegments-1, ppBootUpperArray, ppBootUpperNormal, ppBootDisplacementPigment,0,ppUpperOffSet)
  #declare ppBoot1Upper = ppMeshFromArray(ppBootUpperArray,ppBootUpperNormal,ppUpperSlices,ppSegments,ppJoinEnds)
  #if (ppShowPoints=1) ppDrawPoints(ppBootUpperArray,ppUpperSlices,ppSegments,0.001,<1,0,0>) #end 
  
  // Generate the fabric over the Heel of the Boot
  #if (ppVerbose = 1) #debug concat("Start of Heel","\n")  #end
  ppLimbScan(ppFootObject,ppHeelStartPos,ppHeelStartVector,ppRefVector,ppHeelEndPos,ppHeelEndVector,ppRefVector,ppHeelSlices,
             ppSegments,ppBootHeelArray,ppBootHeelNormal,0,ppRotationDirection,ppTraceLength)
  ppAddOffSetToArray(0, ppHeelSlices-1, 0, ppSegments-1, ppBootHeelArray, ppBootHeelNormal, ppBootDisplacementPigment,0,ppUpperOffSet)
  #declare ppBoot1Heel = ppMeshFromArray(ppBootHeelArray,ppBootHeelNormal,ppHeelSlices,ppSegments,ppJoinEnds)
  #if (ppShowPoints=1) ppDrawPoints(ppBootHeelArray,ppHeelSlices,ppSegments,0.001,<1,1,0>) #end 
  
  // Generate the fabric covering the Leg
  #if (ppVerbose = 1) #debug concat("Start of Boot Core","\n")  #end
  ppLimbScan(ppLegObject,ppCoreStartPos,0,ppRefVector,ppCoreEndPos,0,ppRefVector,ppCoreSlices-1,
             ppSegments,ppBootCoreArray,ppBootCoreNormal,1,ppRotationDirection,ppTraceLength)
  ppStitchArrays(ppBootHeelArray,ppBootHeelNormal, ppHeelSlices-1, ppBootCoreArray,ppBootCoreNormal,0,ppSegments)
  ppAddOffSetToArray(1, 1, 0, ppSegments-1, ppBootCoreArray, ppBootCoreNormal, ppBootDisplacementPigment,ppBootBumpiness*0.85,0)
  ppAddOffSetToArray(2, ppCoreSlices-2, 0, ppSegments-1, ppBootCoreArray, ppBootCoreNormal, ppBootDisplacementPigment,ppBootBumpiness,-ppUpperOffSet/4)
  ppAddOffSetToArray(ppCoreSlices-1, ppCoreSlices-1, 0, ppSegments-1, ppBootCoreArray, ppBootCoreNormal, ppBootDisplacementPigment,0,ppUpperOffSet/4)
  #declare ppBoot1Core = ppMeshFromArray(ppBootCoreArray,ppBootCoreNormal,ppCoreSlices,ppSegments,ppJoinEnds)
  #if (ppShowPoints=1) ppDrawPoints(ppBootCoreArray,ppCoreSlices,ppSegments,0.001,<0,1,1>) #end 
  
  // Generate the Rim around the top of the boot
  #if (ppVerbose = 1) #debug concat("Start of Boot Rim","\n")  #end
  ppLimbScan(ppLegObject,ppRimStartPos,0,ppRefVector,ppRimEndPos,0,ppRefVector,ppRimSlices,
             ppSegments,ppBootRimArray,ppBootRimNormal,1,ppRotationDirection,ppTraceLength)
  ppAddOffSetToArray(1, ppRimSlices, 0, ppSegments-1, ppBootRimArray, ppBootRimNormal, ppBootDisplacementPigment,0,ppUpperOffSet/4)
  ppArrayBand(ppBootRimArray,ppBootRimNormal,4,ppSegments,ppRimThickness)
  #declare ppBoot1Rim = ppMeshFromArray(ppBootRimArray,ppBootRimNormal,4,ppSegments,ppJoinEnds)
  #if (ppShowPoints=1) ppDrawPoints(ppBootRimArray,ppRimSlices+2,ppSegments,0.001,<0,1,0>) #end 
  
  // Generate a Zip down the side of the boot
  #if (ppVerbose = 1) #debug concat("Start of Boot Zip","\n")  #end
  #local ppI = 1;
  #local ppJ = 0;
  #local ppCoreNormal = ppBootCoreArray[ppCoreSlices-1][ppJ]-ppBootCoreArray[0][ppJ];
  #local ppZipDisplacement = ppZipFlapWidth*vaxis_rotate(ppRefVector,ppCoreNormal,ppRotationDirection*80);
  #declare ppBoot1Zip = mesh {
    #while (ppI<ppCoreSlices-1)
      #local ppNextI = ppI+1;
      #local ppPoint1 = ppBootCoreArray[ppI][ppJ]+ppBootCoreNormal[ppI][ppJ]*ppZipFlapThickness;
      #local ppPoint2 = ppBootCoreArray[ppI+1][ppJ]+ppBootCoreNormal[ppI+1][ppJ]*ppZipFlapThickness;
      #local ppPoint3 = ppPoint1+ppZipDisplacement;
      #local ppPoint4 = ppPoint2+ppZipDisplacement;
      #local ppNormal1 = vaxis_rotate(ppBootCoreNormal[ppI][ppJ]  ,ppCoreNormal,-ppRotationDirection*10);
      #local ppNormal2 = vaxis_rotate(ppBootCoreNormal[ppI+1][ppJ],ppCoreNormal,-ppRotationDirection*10);    
      #local ppNormal3 = vaxis_rotate(ppBootCoreNormal[ppI][ppJ]  ,ppCoreNormal, ppRotationDirection*10);
      #local ppNormal4 = vaxis_rotate(ppBootCoreNormal[ppI+1][ppJ],ppCoreNormal, ppRotationDirection*10);    
      smooth_triangle {
        ppPoint1,ppNormal1,    
        ppPoint2,ppNormal2,
        ppPoint3,ppNormal3
      }
      smooth_triangle {
        ppPoint3,ppNormal3,    
        ppPoint2,ppNormal2,
        ppPoint4,ppNormal4
      }
      #local ppI = ppI + 1;
    #end
  }
  
  #local ppToeAndUpper = merge {
    object {ppBoot1Toe   texture {ppBootTexture2}}
    object {ppBoot1Upper texture {ppBootTexture2}}
  }
  
  #if (ppVerbose = 1) #debug concat("Start of Boot Sole","\n")  #end 
  #local ppSegments = 1;
  // Need to move the start point in from the edge a little to avoid the traces missing the upper
  #local ppSoleStartPos = ppUpperStartPos+0.001*(ppUpperEndPos-ppUpperStartPos);
  // Move the end point back into the heel a bit
  #local ppSoleEndPos   = ppUpperEndPos+0.7*(ppUpperEndPos-ppUpperStartPos);
  #local ppHeelAndUpper = union {object {ppBoot1Upper} object {ppBoot1Heel}}
  
  ppLimbScan(ppFootObject,ppSoleStartPos,0,ppPerpRefVector,ppSoleEndPos,0,ppPerpRefVector,ppSoleSlices,
             ppSegments,ppBootSoleArray,ppBootSoleNormal,0,ppRotationDirection,ppTraceLength)
  
  #local ppBootHeelPosition = ppBootSoleArray[ppSoleSlices-1][0];
  #local ppSoleToePosition  = ppBootSoleArray[0][0];
  #local ppTargetOrientation  = ppSoleToePosition-ppBootHeelPosition;
  #local ppASquared = pow(vlength(ppSoleToePosition-ppBootHeelPosition),2)-pow(ppHeelHeight-ppPlatformThickness,2);
  #local ppSourceOrientation  = <0,-(ppHeelHeight-ppPlatformThickness),sqrt(ppASquared)>;
  
  // Calculate the vector of the flat portion of the sole of the boot 
  #local ppRotationPlane = VPerp_To_Plane(ppSourceOrientation, ppTargetOrientation);
  #local ppRotationAngle = VRotationD(ppSourceOrientation, ppTargetOrientation, ppRotationPlane);
  #local ppSoleVector    = vnormalize(vaxis_rotate(z,ppRotationPlane,ppRotationAngle));
  
  //cylinder {ppSoleToePosition,ppSoleToePosition + ppSoleVector,0.001 pigment {color rgb <2,2,1>}}
  //cylinder {ppSoleToePosition,ppSoleToePosition - y,0.001 pigment {color rgb <2,0,0>}}
  
  
  // Create a Sole for our boot
  // Run along the sole of the foot, send rays out sideways (parallel to our Reference Vector) 
  // until they hit the upper. Thicken out a rim and bring a line back into the centre.
  
  #local ppI = 0; 
  #while (ppI<ppSoleSlices) 
    #local ppNorm = <0, 0, 0>; 
    #local ppThisPoint = trace(ppHeelAndUpper, ppBootSoleArray[ppI][0]+ppRefVector, -ppRefVector, ppNorm);
    #if (vlength(ppNorm)!=0)
      // The first array element (0) was set by scanning the foot. The rest are derived by scanning the upper
      #local ppBootSoleNormal[ppI][0] = -ppPerpRefVector;
      #local ppBootSoleArray[ppI][1]  = ppThisPoint+0.1*(ppThisPoint-ppBootSoleArray[ppI][0]);
      #local ppBootSoleNormal[ppI][1] = -ppPerpRefVector;
      #local ppBootSoleArray[ppI][2]  = ppBootSoleArray[ppI][1]; 
      #local ppBootSoleNormal[ppI][2] = ppNorm; //VProject_Plane(ppNorm, ppBootSoleNormal[ppI][0]);
      #local ppBootSoleArray[ppI][3]  = ppBootSoleArray[ppI][1]+ppSoleThickness*vnormalize(ppPerpRefVector);
      #local ppBootSoleNormal[ppI][3] = ppBootSoleNormal[ppI][2]; 
      #local ppBootSoleArray[ppI][4]  = ppBootSoleArray[ppI][3]; 
      #local ppBootSoleNormal[ppI][4] = ppPerpRefVector;
    #else
      #local ppBootSoleArray[ppI][1]  = <0,0,0>;
      #local ppBootSoleNormal[ppI][1] = <0,1,0>;
      #local ppBootSoleArray[ppI][2]  = <0,0,0>;
      #local ppBootSoleNormal[ppI][2] = <0,1,0>;
      #local ppBootSoleArray[ppI][3]  = <0,0,0>;
      #local ppBootSoleNormal[ppI][3] = <0,1,0>;
      #local ppBootSoleArray[ppI][4]  = <0,0,0>;
      #local ppBootSoleNormal[ppI][4] = <0,1,0>;
      #warning concat("The Sole trace sent out a ray from point <",vstr(3,ppBootSoleArray[ppI][0],",",0,3),"> that didn't intercept the boot upper.\n")
    #end        
    #local ppThisPoint = trace(ppHeelAndUpper, ppBootSoleArray[ppI][0]-ppRefVector, ppRefVector, ppNorm);
    #if (vlength(ppNorm)!=0) 
      #local ppBootSoleArray[ppI][9]  = ppThisPoint+0.1*(ppThisPoint-ppBootSoleArray[ppI][0]);
      #local ppBootSoleNormal[ppI][9] = -ppPerpRefVector;
      #local ppBootSoleArray[ppI][8]  = ppBootSoleArray[ppI][9]; 
      #local ppBootSoleNormal[ppI][8] = ppNorm; //VProject_Plane(ppNorm, ppBootSoleNormal[ppI][0]);
      #local ppBootSoleArray[ppI][7]  = ppBootSoleArray[ppI][9]+ppSoleThickness*vnormalize(ppPerpRefVector);
      #local ppBootSoleNormal[ppI][7] = ppBootSoleNormal[ppI][8]; 
      #local ppBootSoleArray[ppI][6]  = ppBootSoleArray[ppI][7]; 
      #local ppBootSoleNormal[ppI][6] = ppPerpRefVector;
    #else
      #local ppBootSoleArray[ppI][9]  = <0,0,0>;
      #local ppBootSoleNormal[ppI][9] = <0,1,0>;
      #local ppBootSoleArray[ppI][8]  = <0,0,0>;
      #local ppBootSoleNormal[ppI][8] = <0,1,0>;
      #local ppBootSoleArray[ppI][7]  = <0,0,0>;
      #local ppBootSoleNormal[ppI][7] = <0,1,0>;
      #local ppBootSoleArray[ppI][6]  = <0,0,0>;
      #local ppBootSoleNormal[ppI][6] = <0,1,0>;
      #warning concat("The Sole trace sent out a ray from point <",vstr(3,ppBootSoleArray[ppI][0],",",0,3),"> that didn't intercept the boot upper.\n")
    #end        
    #local ppThisPoint = trace(ppHeelAndUpper, ppBootSoleArray[ppI][0]+ppPerpRefVector, -ppPerpRefVector, ppNorm);
    #if (vlength(ppNorm)!=0) 
      #local ppBootSoleArray[ppI][5]  = ppThisPoint;
      #local ppBootSoleNormal[ppI][5] = ppPerpRefVector;
    #else
      #local ppBootSoleArray[ppI][5]  = <0,0,0>;
      #local ppBootSoleNormal[ppI][5] = <0,1,0>;
      #warning concat("The Sole trace sent out a ray from point <",vstr(3,ppBootSoleArray[ppI][0],",",0,3),"> that didn't intercept the boot upper.\n")
    #end        
    #local ppI= ppI + 1;
  #end
  
  #declare ppBoot1Sole = ppMeshFromArray(ppBootSoleArray,ppBootSoleNormal,ppSoleSlices,ppSoleSegments,ppJoinEnds)
  #if (ppShowPoints=1) ppDrawPoints(ppBootSoleArray,ppSoleSlices,ppSoleSegments,0.001,<1,0,1>) #end 
  
  
  
  // *************************
  // *************************
  // *************************
  // Taking a point at the base of the front of the foot and a vector that represents the line the sole will take under the toes,
  // send a ray out in an arc to intercept the upper with it's toe section added.
  
  #if (ppVerbose = 1) #debug concat("Start of Toe Section of Boot Sole","\n")  #end 
  #local ppSolePlane = VPerp_To_Plane(ppSoleVector, ppRefVector);
  #local ppI = 1; 
  #while (ppI<ppSoleToeSlices) 
    #local ppBootSoleToeArray[ppI][0] = ppBootSoleArray[0][0] + ppSoleVector*ppRefLength/4.5;
    #local ppNorm = <0, 0, 0>; 
    #local ppRayDirection = vnormalize(vaxis_rotate(ppSoleVector,ppSolePlane,90-((ppI-1)*90/(ppSoleToeSlices-2))));
    #local ppThisPoint = trace(ppBoot1Toe, ppBootSoleToeArray[ppI][0]+ppRayDirection*ppRefLength, -ppRayDirection, ppNorm);
  //cylinder {ppBootSoleToeArray[ppI][0],ppBootSoleToeArray[ppI][0]+ppRayDirection*ppRefLength,0.001  pigment {color rgb <1,0,1>}}
    #if (ppI=1) #local ppProjection = 0.1*vlength(ppThisPoint-ppBootSoleToeArray[ppI][0]); #end
    #if (vlength(ppNorm)!=0)
      #local ppBootSoleToeNormal[ppI][0] = -ppPerpRefVector;
      #local ppBootSoleToeArray[ppI][1]  = ppThisPoint+ppProjection*ppRayDirection;
      #local ppBootSoleToeNormal[ppI][1] = -ppPerpRefVector;
      #local ppBootSoleToeArray[ppI][2]  = ppBootSoleToeArray[ppI][1]; 
      #local ppBootSoleToeNormal[ppI][2] = ppNorm; //VProject_Plane(ppNorm, ppBootSoleToeNormal[ppI][0]);
      #local ppBootSoleToeArray[ppI][3]  = ppBootSoleToeArray[ppI][1]+ppPlatformThickness*vnormalize(ppPerpRefVector);
      #local ppBootSoleToeNormal[ppI][3] = ppBootSoleToeNormal[ppI][2]; 
      #local ppBootSoleToeArray[ppI][4]  = ppBootSoleToeArray[ppI][3]; 
      #local ppBootSoleToeNormal[ppI][4] = ppPerpRefVector;
    #else
      #local ppBootSoleToeNormal[ppI][0] = <0,1,0>;
      #local ppBootSoleToeArray[ppI][1]  = <0,0,0>;
      #local ppBootSoleToeNormal[ppI][1] = <0,1,0>;
      #local ppBootSoleToeArray[ppI][2]  = <0,0,0>;
      #local ppBootSoleToeNormal[ppI][2] = <0,1,0>;
      #local ppBootSoleToeArray[ppI][3]  = <0,0,0>;
      #local ppBootSoleToeNormal[ppI][3] = <0,1,0>;
      #local ppBootSoleToeArray[ppI][4]  = <0,0,0>;
      #local ppBootSoleToeNormal[ppI][4] = <0,1,0>;
      #warning concat("The Sole Toe trace for slice ",str(ppI,3,0)," sent out a ray from point <",vstr(3,ppBootSoleToeArray[ppI][0],",",0,3),"> that didn't intercept the boot upper.\n")
    #end        
    #local ppRayDirection = vnormalize(vaxis_rotate(ppSoleVector,ppSolePlane,-(90-((ppI-1)*90/(ppSoleToeSlices-2)))));
    #local ppThisPoint = trace(ppBoot1Toe, ppBootSoleToeArray[ppI][0]+ppRayDirection*ppRefLength, -ppRayDirection, ppNorm);
  //cylinder {ppBootSoleToeArray[ppI][0],ppBootSoleToeArray[ppI][0]+ppRayDirection*ppRefLength,0.001  pigment {color rgb <1,0,1>}}
    #if (vlength(ppNorm)!=0) 
      #local ppBootSoleToeArray[ppI][9]  = ppThisPoint+ppProjection*ppRayDirection;
      #local ppBootSoleToeNormal[ppI][9] = -ppPerpRefVector;
      #local ppBootSoleToeArray[ppI][8]  = ppBootSoleToeArray[ppI][9]; 
      #local ppBootSoleToeNormal[ppI][8] = ppNorm; //VProject_Plane(ppNorm, ppBootSoleToeNormal[ppI][0]);
      #local ppBootSoleToeArray[ppI][7]  = ppBootSoleToeArray[ppI][9]+ppPlatformThickness*vnormalize(ppPerpRefVector);
      #local ppBootSoleToeNormal[ppI][7] = ppBootSoleToeNormal[ppI][8]; 
      #local ppBootSoleToeArray[ppI][6]  = ppBootSoleToeArray[ppI][7]; 
      #local ppBootSoleToeNormal[ppI][6] = ppPerpRefVector;
    #else
      #local ppBootSoleToeArray[ppI][9]  = <0,0,0>;
      #local ppBootSoleToeNormal[ppI][9] = <0,1,0>;
      #local ppBootSoleToeArray[ppI][8]  = <0,0,0>;
      #local ppBootSoleToeNormal[ppI][8] = <0,1,0>;
      #local ppBootSoleToeArray[ppI][7]  = <0,0,0>;
      #local ppBootSoleToeNormal[ppI][7] = <0,1,0>;
      #local ppBootSoleToeArray[ppI][6]  = <0,0,0>;
      #local ppBootSoleToeNormal[ppI][6] = <0,1,0>;
      #warning concat("The Sole Toe trace for slice ",str(ppI,3,0)," sent out a ray from point <",vstr(3,ppBootSoleToeArray[ppI][0],",",0,3),"> that didn't intercept the boot upper.\n")
    #end        
    #local ppBootSoleToeArray[ppI][5]  = (ppBootSoleToeArray[ppI][4]+ppBootSoleToeArray[ppI][6])/2;
    #local ppBootSoleToeNormal[ppI][5] = ppPerpRefVector;
    #local ppI= ppI + 1;
  #end
  
  ppStitchArrays(ppBootSoleArray,ppBootSoleNormal, 0, ppBootSoleToeArray,ppBootSoleToeNormal,0,ppSoleSegments)
  #declare ppBoot1SoleToe = ppMeshFromArray(ppBootSoleToeArray,ppBootSoleToeNormal,ppSoleToeSlices,ppSoleSegments,ppJoinEnds)
  #if (ppShowPoints=1) ppDrawPoints(ppBootSoleToeArray,ppSoleToeSlices,ppSoleSegments,0.001,<0,3,0>) #end 
  
  // Create a high heel using CSG
  #if (ppVerbose = 1) #debug concat("Start of Boot Heel","\n")  #end 
  #declare ppHighHeel = difference {
    difference {
      box {<-ppHeelWidth/1.5,0,-ppHeelWidth/1.5><ppHeelWidth/1.5,ppHeelHeight,ppHeelWidth*0.9>}
      torus {ppHeelHeight*1.05,ppHeelHeight scale <0.8,1.45,1.2> rotate x*8}
      cylinder {<-ppHeelWidth,0,0><ppHeelWidth,0,0>,ppHeelHeight scale <1,1,0.58> translate 0.6*ppHeelHeight*z
      texture {ppBootHeelInnerTexture}}
      rotate -x*6 scale <1.8,1.1,1.5>
      translate <0,-ppHeelHeight,ppHeelWidth/5>
      Reorient_Trans(ppSourceOrientation, ppTargetOrientation)
    }
    plane {-y,0}
  }
  
  
  #declare ppBoot1 = union {
    object {ppBoot1Core    texture {ppBootLegTexture rotate <ppXRot,0,ppZRot>}}  
    object {ppBoot1Rim     texture {ppBootRimTexture}}
    object {ppBoot1Zip     texture {ppBootZipTexture}}
    object {ppBoot1Toe     texture {ppBootToeTexture}}
    object {ppBoot1Upper   texture {ppBootUpperTexture}}
    object {ppBoot1Heel    texture {ppBootUpperTexture}}
    object {ppHighHeel     texture {ppBootHeelOuterTexture} translate ppBootHeelPosition}
    object {ppBoot1Sole    texture {ppBootSoleTexture}}
    object {ppBoot1SoleToe texture {ppBootSoleTexture}}
  }
  
#end

ppFootwear()