/***************************************************************************************************/
/* pplips.inc  - Copyright Chris Bartlett 3 Mar 2005                                               */
/* This work is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike License.  */
/* To view a copy of this license, visit http://creativecommons.org/licenses/by-nc-sa/1.0/ or      */
/* send a letter to Creative Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.      */
/***************************************************************************************************/ 

//  xxxxxxxx  Bug.  Currently the ppMesh2FromArray macro overwrites the normals read from the lipline CSV files.  xxxxxxxxxx

#ifndef(ppCamera)   #declare ppCamera="Lips";       #end
#ifndef(POVPerson)  #include "povperson.inc"        #end 
#ifndef(ppLimbScan) #include "ppclothingmacros.inc" #end 

#declare ppVerbose=0;
#declare ppShowPoints=0;
#ifndef (ppLipTexture) #declare ppLipTexture  = texture {pigment {color rgb <1,0,0>}} #end
#declare ppRadius = 0.1; 

//  Imagine the lip sliced length ways, then each slice divided into segments along the length.
#local ppLipSlices   = 9;
#local ppMaxLipSegments = 30;
#local ppUpperLipSegments = ppMaxLipSegments;   // Will be adjusted when we read the file and find out how many there really are.
#local ppLowerLipSegments = ppMaxLipSegments;   // Will be adjusted when we read the file and find out how many there really are.
#local ppkDontJoinEnds = 0;

#ifndef (ppUpperLipLineFile) #declare ppUpperLipLineFile = "ppupperlipline.csv";  #end
#ifndef (ppLowerLipLineFile) #declare ppLowerLipLineFile = "pplowerlipline.csv";  #end

#declare ppUpperLipArray  = array[ppLipSlices][ppMaxLipSegments]; 
#declare ppUpperLipNormal = array[ppLipSlices][ppMaxLipSegments]; 
#declare ppLowerLipArray  = array[ppLipSlices][ppMaxLipSegments]; 
#declare ppLowerLipNormal = array[ppLipSlices][ppMaxLipSegments]; 


#if (ppVerbose = 1) #debug concat("Start of Upper Lip","\n")  #end 
#declare ppLipProfile = array[ppLipSlices-1] {
  <0,-0.1, 0.0>,
  <0,-0.5, 0.5>,
  <0,-0.5, 1.0>,
  <0,-0.5, 1.5>,
  <0,-0.5, 2.0>,
  <0, 0.5, 2.0>,
  <0, 0.5, 1.0>,
  <0, 0.5, 0.0>
}
// Read the line of the upper lip into the first slice of a pair of arrays, then extrude along a profile.
ppReadLineFromCSV (ppUpperLipLineFile, ppUpperLipArray, ppUpperLipNormal, ppUpperLipSegments)
ppExtrudeLipAlongProfile (ppUpperLipArray, ppUpperLipNormal, ppLipSlices, ppUpperLipSegments, ppLipProfile,-1)
#declare ppUpperLip = ppMesh2FromArray(ppUpperLipArray,ppUpperLipNormal,ppLipSlices,ppUpperLipSegments,ppkDontJoinEnds)


#if (ppVerbose = 1) #debug concat("Start of Lower Lip","\n")  #end 
#declare ppLipProfile = array[ppLipSlices-1] {
  <0, 0.1, 0.0>,
  <0, 0.5, 0.5>,
  <0, 0.5, 1.0>,
  <0, 0.5, 1.5>,
  <0, 0.5, 2.0>,
  <0,-0.5, 2.0>,
  <0,-0.5, 1.0>,
  <0,-0.5, 0.0>
}
//  Read the line of the lower lip into the first slice of a pair of arrays, then extrude it along a profile.
ppReadLineFromCSV (ppLowerLipLineFile, ppLowerLipArray, ppLowerLipNormal, ppLowerLipSegments)
ppExtrudeLipAlongProfile (ppLowerLipArray, ppLowerLipNormal, ppLipSlices, ppLowerLipSegments, ppLipProfile, 1)
#declare ppLowerLip = ppMesh2FromArray(ppLowerLipArray,ppLowerLipNormal,ppLipSlices,ppLowerLipSegments,ppkDontJoinEnds)


#declare ppLips = union {object {ppUpperLip} object {ppLowerLip}} 


#if (strcmp(ppCamera,"Lips")=0)  
  #declare ppShowPoints = 1;
  #debug concat("Lip: ",vstr(3,ppUpperLipArray[0][0],",",0,3) ,"\n")         
  #if (ppShowPoints=1) ppDrawPoints(ppUpperLipArray,ppLipSlices,ppUpperLipSegments,0.1,<0,3,3>) #end    

  #local ppMin = min_extent (ppLips);
  #local ppMax = max_extent (ppLips);
  #local ppCenter = ppMin + (ppMax-ppMin)/2;


  object {ppUpperLip texture {ppLipTexture}}
  object {ppLowerLip texture {ppLipTexture}}

  light_source { <2, 60,-10> color rgb<1, 1, 1>*1.4 }   
  light_source { < 100, 14,-100> color rgb<1, 1, 1>*1.2 }   
  light_source { <-100, 14,10> color rgb<1, 1, 1>*1.2 }   
//  light_source { <-10, 1,-1> color rgb<1, 0.5, 0.5>}   
//  light_source { <-0.7,0.7,0.2> color rgb<1, 1, 1>*1 }   
  camera {location ppCenter-30*z look_at ppCenter} // look_at <11,56,-16>}
#end   
